#include <math.h>
#include <mex.h>

#include <stdio.h>


#include <cv.h>
#include <highgui.h>




/*

function [Rect]=HaarDetection(Image,CascadeName)

Receives:
	Image: RGBColor image or Gray Level
	CascadeName: Path to HaarTrainning Cascade p.e:'haarcascade_frontalface_alt.xml'

Returns:
	Rect: [x,y,width,height] size [NDetections x 4] -> Rectangle Data for Each Detection 
		- (x,y) is the Upper Right Rectangle Coorner
		- Each Rect rows contains Rectangle data for a Detection


Example of Usage:

Image=imread('lena.jpg');
Face=HaarDetection(Image,'haarcascade_frontalface_alt.xml')
Face =

         207         198         180         180
*/

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){

	/*nlhs -> outputs*/	
	/*nrhs -> inputs*/

	unsigned char*MatLabImage;
	IplImage*OpenCVImage=NULL;
	int ImageInputChannels;

	char*CascadeName;
	int BufferLen;

	const int *Size;
	int i,j,k;


	CvMemStorage*Storage = 0;
	CvHaarClassifierCascade*Cascade = 0;

	IplImage*GrayImage=NULL,*SmallImage=NULL;
	int scale = 3;
	CvRect*r;
	CvSeq* Faces;

	int*OutputRect;
	int nRows;

	
	/*Check Input/OutPut Parameters*/
	if(nrhs!=2 || nlhs > 1) mexErrMsgTxt("Format: [Rect]=HaarDetection(RGBImage,CascadeName)");

	/*Second Input must be a string. */
	if (!mxIsChar(prhs[1])) mexErrMsgTxt("CascadeName Should be a String");
	

	/***************************/
	/*Setup Cascade String Name*/
	/***************************/

		/*Get the length of the input string [prhs[1]]*/
		BufferLen=(mxGetM(prhs[1])*mxGetN(prhs[1]))+1;
		
		/*Allocate memory for CascadeName strings*/
		CascadeName=mxCalloc(BufferLen, sizeof(char));
	
		/*Copy the string data from prhs[1] into a C string CascadeName*/
		mxGetString(prhs[1],CascadeName,BufferLen);
		/*printf("Cascade Namer=%s\n",CascadeName);*/


		/*Load Cascade*/
		Cascade = (CvHaarClassifierCascade*)cvLoad(CascadeName, 0, 0, 0);
		if(!Cascade ) mexErrMsgTxt("HaarDetection: Could Not Load Classifier Cascade\n");
	
		/*Create Memory Storage*/
		Storage = cvCreateMemStorage(0);



	/*************/
	/*Setup Image*/
	/*************/

	/*Get Image Size*/
	Size=mxGetDimensions(prhs[0]); 
	/*printf("Matlab Image Size %d %d %d\n",Size[0],Size[1],Size[2]);*/
	/*printf("Ndim=%d\n",mxGetNumberOfDimensions(prhs[0]));*/

	/*Check Input Image Number of Channels*/
	/*ImageInputChannels=2 for OneChannel Image and ImageInputChannels=3 for Color Images*/  
	ImageInputChannels=mxGetNumberOfDimensions(prhs[0]);
	if(ImageInputChannels<3) ImageInputChannels=1;

	/*Create Image*/
	OpenCVImage=cvCreateImage(cvSize(Size[1],Size[0]),IPL_DEPTH_8U,ImageInputChannels);

	/*Assign MatLabImage to prhs[0] Input Data*/
	MatLabImage =(unsigned char*)mxGetData(prhs[0]);

	/*Convert MatLab Image matrix into IplImage Data*/
	for(i=0;i<OpenCVImage->height;i++)
		for(j=0;j<OpenCVImage->width;j++) 
			for(k=0;k<OpenCVImage->nChannels;k++) 
				OpenCVImage->imageData[i*OpenCVImage->widthStep+j*OpenCVImage->nChannels+k] = MatLabImage[(i+j*OpenCVImage->height)+(OpenCVImage->width*OpenCVImage->height*k)];





	
	/*Alloc Image Data*/	
	SmallImage=cvCreateImage(cvSize(cvRound(OpenCVImage->width/scale),cvRound(OpenCVImage->height/scale)),IPL_DEPTH_8U,1);

	/*Use an Extra Gray Level > Image for Color Image Inputs*/
	if(ImageInputChannels>2){
				GrayImage=cvCreateImage(cvSize(OpenCVImage->width,OpenCVImage->height),IPL_DEPTH_8U,1);
				cvCvtColor(OpenCVImage,GrayImage,CV_RGB2GRAY);
				cvResize(GrayImage,SmallImage,CV_INTER_LINEAR);
	}
	else  cvResize(OpenCVImage,SmallImage,CV_INTER_LINEAR);

	cvEqualizeHist(SmallImage,SmallImage);
	cvClearMemStorage(Storage);



	/*Boosted Haar classifier Detector*/ 
	Faces = cvHaarDetectObjects(SmallImage,Cascade,Storage,1.1,2,1/*CV_HAAR_DO_CANNY_PRUNING*/,cvSize(30,30));



	/************/
	/*Set Output*/
	/************/

	/*No Faces Not Found*/
	if(Faces->total==0) {

		/*Alloc Memory for Output Data*/
		plhs[0] = mxCreateNumericMatrix(1,4,mxINT32_CLASS,mxREAL);
	
		/*Get a pointer to the data space in our newly allocated memory*/
		OutputRect = (int *)mxGetData(plhs[0]);

		for (i=0;i<4;i++) OutputRect[i]=-1;
	}


	else {
		nRows=Faces->total;

		/*Alloc Memory for Output Data*/
		plhs[0] = mxCreateNumericMatrix(nRows,4,mxINT32_CLASS,mxREAL);
	
		/*Get a pointer to the data space in our newly allocated memory*/
		OutputRect = (int *)mxGetData(plhs[0]);


		/* 
		i=1; j=1;	
		OutputRect[i+j*nRows]=1000;*/

	
		for(i=0;i<(Faces ? Faces->total:0);i++){
			r = (CvRect*)cvGetSeqElem(Faces,i);
			/*printf("Rect %d %d %d %d\n",scale*r->x,scale*r->y,scale*r->width,scale*r->height);*/


			/*OutPutRect With Format: [x,y,width,height] Per Row of a Detected Face*/
			OutputRect[i]=scale*r->x;
			OutputRect[i+nRows]=scale*r->y;
			OutputRect[i+2*nRows]=scale*r->width;
			OutputRect[i+3*nRows]=scale*r->height;
		}
	
	}





	/*Release Data*/
	cvReleaseImage(&OpenCVImage);
	cvReleaseImage(&GrayImage);
	cvReleaseImage(&SmallImage);

}

